# Generated by Django 4.2.19 on 2025-02-21 03:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("prepayments", "0002_projectprepaymentdetail_overdraft_amount_allowed"),
    ]

    operations = [
        migrations.AlterField(
            model_name="fundtype",
            name="name",
            field=models.CharField(help_text="The unique name for this item", max_length=255, unique=True),
        ),
    ]
