# Generated by Django 3.2.16 on 2023-01-13 23:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_billing", "0002_version_1_2_0"),
    ]

    operations = [
        migrations.AddField(
            model_name="customcharge",
            name="cap_eligible",
            field=models.BooleanField(default=False, help_text="Check this box to make this charge count towards CAP"),
        ),
    ]
