{% load static %}
{% load custom_tags_and_filters %}
{% if user.is_any_part_of_staff %}
    {% navigation_url 'custom_charges' 'Custom charges' user.is_user_office user.is_accounting_officer user.is_facility_manager user.is_superuser %}
    {% navigation_url 'invoices' 'Invoices' user.is_user_office user.is_accounting_officer user.is_facility_manager user.is_superuser %}
    {% navigation_url 'rates' 'Rates' user.is_accounting_officer user.is_facility_manager user.is_superuser %}
    {% if "NEMO_billing.prepayments"|app_installed and "NEMO_billing.cap_discount"|app_installed %}
        {% navigation_url 'prepaid_project_status' 'CAP and prepaid status' user.is_accounting_officer user.is_user_office user.is_facility_manager user.is_superuser %}
    {% elif "NEMO_billing.prepayments"|app_installed %}
        {% navigation_url 'prepaid_project_status' 'Prepaid project status' user.is_accounting_officer user.is_user_office user.is_facility_manager user.is_superuser %}
    {% elif "NEMO_billing.cap_discount"|app_installed %}
        {% navigation_url 'cap_discount_status' 'CAP status' user.is_accounting_officer user.is_user_office user.is_facility_manager user.is_superuser %}
    {% endif %}
{% endif %}
