# Generated by Django 4.2.11 on 2024-08-27 15:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_billing", "0006_projectbillinghardcap"),
    ]

    operations = [
        migrations.AlterField(
            model_name="projectbillinghardcap",
            name="end_date",
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="projectbillinghardcap",
            name="start_date",
            field=models.DateField(blank=True, null=True),
        ),
    ]
