# Generated by Django 4.2.23 on 2025-11-19 17:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cap_discount", "0005_cap_multi_core_facilities"),
    ]

    operations = [
        migrations.AddField(
            model_name="capdiscount",
            name="creation_time",
            field=models.DateTimeField(null=True),
        ),
        migrations.AlterField(
            model_name="capdiscount",
            name="creation_time",
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
        migrations.AddField(
            model_name="capdiscountconfiguration",
            name="creation_time",
            field=models.DateTimeField(null=True),
        ),
        migrations.AlterField(
            model_name="capdiscountconfiguration",
            name="creation_time",
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
    ]
