# Generated by Django 3.2.15 on 2022-11-12 18:51

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("invoices", "0008_version_1_10_0"),
    ]

    operations = [
        migrations.AddField(
            model_name="invoicedetailitem",
            name="content_type",
            field=models.ForeignKey(
                blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to="contenttypes.contenttype"
            ),
        ),
        migrations.AddField(
            model_name="invoicedetailitem",
            name="object_id",
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddIndex(
            model_name="invoicedetailitem",
            index=models.Index(fields=["content_type", "object_id"], name="invoices_in_content_622541_idx"),
        ),
        migrations.AddField(
            model_name="invoiceconfiguration",
            name="hide_zero_charge",
            field=models.BooleanField(default=True, help_text="Hide charges with an amount equal to 0."),
        ),
    ]
