# Generated by Django 4.2.23 on 2026-01-07 14:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("rates", "0010_rate_service_fee"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="rate",
            options={"ordering": ["type", "category", "time", "effective_date"]},
        ),
    ]
