# Generated by Django 4.2.23 on 2026-01-07 14:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cap_discount", "0006_capdiscount_creation_time_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="capdiscount",
            options={"ordering": ["-creation_time", "configuration", "account", "user"]},
        ),
    ]
