{% load custom_tags_and_filters %}
<div class="panel-body">
    <h3 class="customization-section-title" id="quote_email_id">Quote Email</h3>
    <p>This email is sent to the quote recipients with a link to the quote PDF file.</p>
    <p>The following context variables are provided when the email is rendered:</p>
    <ul>
        <li>
            <b>quote</b> - the quote object
        </li>
        <li>
            <b>quote_url</b> - the url to the quote PDF
        </li>
    </ul>
    {% include 'customizations/customizations_upload.html' with element=quote_email name='quote email' key='quotes' %}
    <div class="customization-separation"></div>
</div>
<div class="panel-body">
    <h3 class="customization-section-title" id="quote_approval_request_email_id">
        Quote Approval Request Notification Email
    </h3>
    <p>This email is sent to users with the "approval" permission when a user submits a quote approval request.</p>
    <p>The following context variables are provided when the email is rendered:</p>
    <ul>
        <li>
            <b>quote</b> - the quote object
        </li>
    </ul>
    {% include 'customizations/customizations_upload.html' with element=quote_approval_request_email name='quote approval request email' key='quotes' %}
    <div class="customization-separation"></div>
</div>
<div class="panel-body">
    <h3 class="customization-section-title" id="quote_status_update_email_id">Quote Status Update Notification Email</h3>
    <p>This email is sent to the quote creator when a quote is approved or denied.</p>
    <p>The following context variables are provided when the email is rendered:</p>
    <ul>
        <li>
            <b>quote</b> - the quote object
        </li>
    </ul>
    {% include 'customizations/customizations_upload.html' with element=quote_status_update_email name='quote status update email' key='quotes' %}
</div>
