# Generated by Django 4.2.11 on 2024-07-31 22:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO", "0086_adjustmentrequest_new_quantity"),
        ("rates", "0006_rate_daily_split_multi_day_charges"),
    ]

    operations = [
        migrations.CreateModel(
            name="AreaHighestDailyRateGroup",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "areas",
                    models.ManyToManyField(
                        help_text="Select all areas that require grouping daily, and for which only the most expensive one will be charged.",
                        to="NEMO.area",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
