# Generated by Django 3.2.16 on 2023-02-07 00:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0009_version_1_23_0"),
    ]

    operations = [
        migrations.AddField(
            model_name="invoicedetailitem",
            name="discount",
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=14, null=True),
        ),
    ]
