{% extends 'pagination/pagination_base.html' %}
{% block extrahead %}
    <style>
		.panel-body .pagination:last-of-type
        {
			margin-bottom: 0;
		}
	
    </style>
{% endblock %}
{% load custom_tags_and_filters %}
{% block content %}
    <h1 style="margin-top: 0; margin-bottom: 25px">
        {% block tab_title %}{% endblock %}
    </h1>
    {% url 'prepaid_project_status' as prepaid_status_url %}
    {% url 'cap_discount_status' as cap_discount_status_url %}
    {% if prepaid_status_url and cap_discount_status_url %}
        <ul class="nav nav-tabs">
            <li class="nav-item{% if prepaid_status_url in request.path %} active{% endif %}">
                <a class="nav-link" href="{{ prepaid_status_url }}">Prepaid status</a>
            </li>
            <li class="nav-item{% if cap_discount_status_url in request.path %} active{% endif %}">
                <a class="nav-link" href="{{ cap_discount_status_url }}">CAP status</a>
            </li>
        </ul>
        <div id="contract-panel-content"
             class="tab-content panel panel-default panel-tab-content info-tooltip-container">
            <div class="panel-body">
            {% endif %}
            {{ block.super }}
            {% if prepaid_status_url and cap_discount_status_url %}
            </div>
        </div>
    {% endif %}
{% endblock %}
