# Generated by Django 3.2.19 on 2023-06-04 18:53

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_billing", "0004_version_2_4_0"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("invoices", "0010_version_2_0_0"),
    ]

    operations = [
        migrations.AddField(
            model_name="projectbillingdetails",
            name="department",
            field=models.ForeignKey(
                blank=True,
                help_text="The project department",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="NEMO_billing.department",
            ),
        ),
        migrations.AddField(
            model_name="projectbillingdetails",
            name="institution",
            field=models.ForeignKey(
                blank=True,
                help_text="The project institution",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="NEMO_billing.institution",
            ),
        ),
        migrations.AddField(
            model_name="projectbillingdetails",
            name="staff_host",
            field=models.ForeignKey(
                blank=True,
                help_text="The project host",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]
