{% extends 'base.html' %}
{% block extrahead %}
    {% load static %}
    <link rel="stylesheet" type="text/css" href="{% static "invoices/invoices.css" %}" />
    {% block invoiceextrahead %}{% endblock %}
{% endblock %}
