# Generated by Django 4.2.15 on 2024-10-05 21:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0013_alter_invoiceconfiguration_name_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="invoicedetailitem",
            name="waived",
            field=models.BooleanField(default=False),
        ),
    ]
