# Generated by Django 4.2.20 on 2025-04-28 14:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("rates", "0008_alter_ratecategory_name"),
    ]

    def forward(apps, schema_editor):
        Customization = apps.get_model("NEMO", "Customization")
        Customization.objects.filter(name="allow_blank_rate").delete()

    def reverse(apps, schema_editor):
        pass

    operations = [
        migrations.AddField(
            model_name="rate",
            name="effective_date",
            field=models.DateField(blank=True, help_text="The date when this rate becomes effective.", null=True),
        ),
        migrations.RunPython(forward, reverse),
    ]
