# Generated by Django 4.2.23 on 2025-10-03 16:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("quotes", "0002_quote_add_tax_and_number"),
    ]

    operations = [
        migrations.AddField(
            model_name="quoteitem",
            name="service_fee",
            field=models.DecimalField(
                blank=True, decimal_places=2, help_text="Service fee for this item", max_digits=8, null=True
            ),
        ),
    ]
