# Generated by Django 2.2.21 on 2021-06-07 02:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0003_version_1_4_0"),
    ]

    operations = [
        migrations.AddField(
            model_name="projectbillingdetails",
            name="contact_phone",
            field=models.CharField(blank=True, help_text="The contact's phone number", max_length=40, null=True),
        ),
    ]
