# Generated by Django 4.2.23 on 2025-11-07 20:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0014_invoicedetailitem_waived"),
    ]

    operations = [
        migrations.AddField(
            model_name="projectbillingdetails",
            name="no_cap",
            field=models.BooleanField(
                default=False, help_text="Check this box if this project should not count towards CAP."
            ),
        ),
    ]
