# Generated by Django 2.2.20 on 2021-04-22 12:21

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO", "0027_version_3_8_0"),
        ("NEMO_billing", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="corerelationship",
            name="staff_charge",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="core_rel",
                to="NEMO.StaffCharge",
            ),
        ),
    ]
