.invoice-summary-facility.invoice-summary-item .summary-item-name,
.invoice-summary-facility.invoice-summary-sub_total .summary-item-name,
.invoice-summary-facility.invoice-summary-discount .summary-item-name,
.invoice-summary-facility.invoice-summary-discount_sub_total .summary-item-name
{
    padding-left: 25px;
}

.invoice-summary-facility.invoice-summary-discount_sub_total
{
    display: none;
}

.invoice-summary-category .summary-item-name,
.invoice-summary-sub_total .summary-item-name,
.invoice-summary-discount .summary-item-name,
.invoice-summary-fund .summary-item-name,
.invoice-summary-tax .summary-item-name
{
    font-weight: bold;
}

.detail-item-amount, .summary-item-amount
{
    text-align: right;
}

ul.invoice-payment-list li
{
    margin-top: 7px;
    margin-bottom: 7px;
}

table.invoice-details-table thead th, table.invoice-summary-table thead th
{
    background-color: #dbdbdb;
}

.table-display
{
    display: table;
    width: 100%;
}

.table-cell-display
{
    display: table-cell;
    vertical-align: middle;
    float: none !important;
}