# Generated by Django 3.2.25 on 2024-05-27 15:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0011_version_2_4_0"),
    ]

    operations = [
        migrations.AddField(
            model_name="invoiceconfiguration",
            name="invoice_title",
            field=models.CharField(
                default="Invoice",
                help_text="The title displayed on the first page of the invoice (e.g. Invoice/Statement/Quote)",
                max_length=255,
            ),
        ),
    ]
