# Generated by Django 3.2.19 on 2023-06-02 00:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("rates", "0004_version_2_0_0"),
    ]

    operations = [
        migrations.AddField(
            model_name="rate",
            name="daily",
            field=models.BooleanField(
                default=False, help_text="Check this box to only charge once per day for this item"
            ),
        ),
    ]
