{% extends 'billing/cap_and_prepaid_base.html' %}
{% load custom_tags_and_filters %}
{% block title %}Prepaid project status{% endblock %}
{% block tab_title %}Prepaid project status{% endblock %}
{% block before_pagination %}
    <form class="form-inline" action="{% url 'prepaid_project_status' %}" method="get">
        <input type="hidden" name="o" value="{{ request.GET.o }}">
        <label for="start" style="padding: 10px 10px 10px 0">Since</label>
        <select name="start" id="start" class="form-control" style="margin-right: 5px;text-align-last:center">
            {% for month in month_list %}
                <option value="{{ month.date|input_date_format }}"
                        {% if month|date:"F, Y" == start.date|date:"F, Y" %}selected{% endif %}>
                    {{ month|date:"F, Y" }}
                </option>
            {% endfor %}
        </select>
        {% button type="save" value="Update" %}
    </form>
{% endblock %}
{% block pagination_content %}
    {% include 'prepayments/prepaid_project_status_table.html' %}
{% endblock %}
{% block table_empty_content %}There are no prepaid projects set{% endblock %}
