# Generated by Django 4.2.20 on 2025-05-23 17:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_billing", "0008_customcharge_validated_customcharge_validated_by_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="corefacility",
            name="external_id",
            field=models.CharField(
                blank=True, null=True, help_text="An external ID to associate with this core facility.", max_length=255
            ),
        ),
    ]
