<html lang="en">
    <head>
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1">
    </head>
    <body style="font-family: 'Avenir Next', 'Helvetica Neue', 'Helvetica', 'Arial', 'sans-serif'">
        <!--[if mso]><table width="600" align="center"><tr><td><![endif]-->
        <table align="center" style="width:100%; max-width:600px">
            <tr>
                <td align="center"
                    style="color: white;
                           background: #5bc0de;
                           margin: 0;
                           padding: 50px 0"
                    bgcolor="#5bc0de">
                    <h1 style="max-width: 90%; margin: 0 auto; padding: 0;">Invoice {{ invoice.invoice_number }} past due!</h1>
                </td>
            </tr>
            <tr>
                <td style="margin: 10px 10px 20px; padding: 10px;">
                    {% if invoice.project_details.contact_name %}<h2>Dear {{ invoice.project_details.contact_name }},</h2>{% endif %}
                    <p>
                        This is a reminder that your {{ invoice.configuration.merchant_name }} invoice for the period of <b>{{ invoice.start|date:'F jS, Y' }}</b> to <b>{{ invoice.end|date:'F jS, Y' }}</b> for your project {{ invoice.project_details.name }} is past due.
                        <br>
                        <br>
                        The remaining amount of <b>{{ invoice.total_outstanding_display }}</b> is due immediately.
                        <br>
                        <br>
                        Please refer to the payment terms and conditions included in the invoice.
                    </p>
                </td>
            </tr>
        </table>
        <!--[if mso]></td></tr></table><![endif]-->
    </body>
</html>
