# Generated by Django 2.2.20 on 2021-05-02 11:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0002_version_1_2_0"),
    ]

    operations = [
        migrations.RenameField(
            model_name="projectbillingdetails",
            old_name="details",
            new_name="addressee",
        ),
        migrations.AlterField(
            model_name="projectbillingdetails",
            name="addressee",
            field=models.TextField(
                blank=True, help_text="The addressee details to be included in the invoice", null=True
            ),
        ),
        migrations.AddField(
            model_name="projectbillingdetails",
            name="comments",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="projectbillingdetails",
            name="expires_on",
            field=models.DateField(
                blank=True, help_text="Date after which this project will be automatically deactivated.", null=True
            ),
        ),
    ]
