# Generated by Django 4.2.15 on 2024-10-05 13:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("rates", "0007_areahighestdailyrategroup"),
    ]

    operations = [
        migrations.AlterField(
            model_name="ratecategory",
            name="name",
            field=models.CharField(help_text="The name of this rate category", max_length=255, unique=True),
        ),
    ]
