# Generated by Django 3.2.16 on 2023-01-28 19:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("rates", "0003_version_1_17_0"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="ratecategory",
            options={"ordering": ["name"], "verbose_name": "Rate category", "verbose_name_plural": "Rate categories"},
        ),
    ]
