# Generated by Django 4.2.15 on 2024-10-05 13:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0012_invoiceconfiguration_invoice_title"),
    ]

    operations = [
        migrations.AlterField(
            model_name="invoiceconfiguration",
            name="name",
            field=models.CharField(help_text="The name of this invoice configuration", max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="invoiceconfiguration",
            name="tax_name",
            field=models.CharField(blank=True, default="VAT", max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name="invoicedetailitem",
            name="user",
            field=models.CharField(max_length=255),
        ),
        migrations.AlterField(
            model_name="projectbillingdetails",
            name="project_name",
            field=models.CharField(
                blank=True,
                help_text="The project name that will appear on the invoices. Leave blank to use NEMO project name",
                max_length=1024,
                null=True,
            ),
        ),
    ]
