# Generated by Django 4.2.20 on 2025-08-09 17:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("quotes", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="quote",
            name="add_tax",
            field=models.BooleanField(default=False, help_text="Whether to add tax to the total amount of this quote"),
        ),
        migrations.AddField(
            model_name="quote",
            name="quote_number",
            field=models.CharField(
                blank=True,
                help_text="Quote number. This is automatically generated based on the quote configuration",
                max_length=255,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="quoteconfiguration",
            name="current_quote_number",
            field=models.PositiveIntegerField(
                default=0,
                help_text="The current quote number used for this configuration. This is automatically incremented when a new quote is created.",
            ),
        ),
        migrations.AddField(
            model_name="quoteconfiguration",
            name="quote_numbering_template",
            field=models.CharField(
                default="{{ number|stringformat:'04d' }}",
                help_text="Django template for quote numbering. Available variables: date_created, number",
                max_length=255,
            ),
        ),
    ]
