# Generated by Django 4.2.23 on 2025-12-15 22:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("quotes", "0003_quoteitem_service_fee"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="quoteconfiguration",
            name="email_from",
        ),
    ]
