# Generated by Django 2.2.20 on 2021-04-21 18:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="invoiceconfiguration",
            name="separate_area_access_charges",
            field=models.BooleanField(
                default=False, help_text="Check this box to display area access charges as a separate invoice category"
            ),
        ),
        migrations.AddField(
            model_name="invoiceconfiguration",
            name="separate_tool_usage_charges",
            field=models.BooleanField(
                default=False, help_text="Check this box to display tool usage charges as a separate invoice category"
            ),
        ),
        migrations.AddField(
            model_name="invoiceconfiguration",
            name="separate_consumable_charges",
            field=models.BooleanField(
                default=False, help_text="Check this box to display supplies/material as a separate invoice category"
            ),
        ),
        migrations.AddField(
            model_name="invoiceconfiguration",
            name="separate_custom_charges",
            field=models.BooleanField(
                default=False, help_text="Check this box to display custom charges as a separate invoice category"
            ),
        ),
        migrations.AddField(
            model_name="invoiceconfiguration",
            name="separate_missed_reservation_charges",
            field=models.BooleanField(
                default=False,
                help_text="Check this box to display missed reservation charges as a separate invoice category",
            ),
        ),
        migrations.AddField(
            model_name="invoiceconfiguration",
            name="separate_staff_charges",
            field=models.BooleanField(
                default=False, help_text="Check this box to display staff charges as a separate invoice category"
            ),
        ),
        migrations.AddField(
            model_name="invoiceconfiguration",
            name="separate_training_charges",
            field=models.BooleanField(
                default=False, help_text="Check this box to display training charges as a separate invoice category"
            ),
        ),
        migrations.AddField(
            model_name="invoicesummaryitem",
            name="item_type",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (1, "tool_usage"),
                    (2, "area_access"),
                    (3, "consumable"),
                    (4, "missed_reservation"),
                    (5, "staff_charge"),
                    (6, "training_session"),
                    (7, "custom_charge"),
                ],
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="invoicesummaryitem",
            name="summary_item_type",
            field=models.IntegerField(
                choices=[
                    (1, "item"),
                    (2, "sub_total"),
                    (3, "discount_sub_total"),
                    (4, "discount"),
                    (5, "tax"),
                    (6, "other"),
                    (7, "fund"),
                ]
            ),
        ),
    ]
