# Generated by Django 4.2.23 on 2025-10-03 16:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("rates", "0009_rate_effective_date"),
    ]

    operations = [
        migrations.AddField(
            model_name="rate",
            name="service_fee",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="The service fee for this rate (always applies)",
                max_digits=8,
                null=True,
            ),
        ),
    ]
