# Generated by Django 4.2.15 on 2024-10-05 13:39

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("NEMO_billing", "0007_alter_projectbillinghardcap_end_date_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="customcharge",
            name="validated",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="customcharge",
            name="validated_by",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="custom_charge_validated_set",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AddField(
            model_name="customcharge",
            name="waived",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="customcharge",
            name="waived_by",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="custom_charge_waived_set",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AddField(
            model_name="customcharge",
            name="waived_on",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="corefacility",
            name="name",
            field=models.CharField(help_text="The name of this core facility.", max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="department",
            name="name",
            field=models.CharField(help_text="The unique name for this item", max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="institutiontype",
            name="name",
            field=models.CharField(help_text="The unique name for this item", max_length=255, unique=True),
        ),
    ]
