# Generated by Django 2.2.24 on 2021-09-02 19:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0006_version_1_7_1"),
    ]

    operations = [
        migrations.AddField(
            model_name="projectbillingdetails",
            name="no_tax",
            field=models.BooleanField(default=False, help_text="Check this box if this project is tax exempt."),
        ),
    ]
