# Generated by Django 2.2.24 on 2021-09-14 14:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0007_version_1_8_0"),
    ]

    operations = [
        migrations.AddField(
            model_name="invoicepayment",
            name="note",
            field=models.CharField(blank=True, help_text="Payment note", max_length=255, null=True),
        ),
    ]
